#include <stdlib.h>

#include <qtranslator.h>
#include <qapplication.h>
#include <qfile.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include <qstylefactory.h>

#include "init.h"
#include "firststartImpl.h"

extern QApplication * appTipo;
extern QTranslator  * transTipo;

// ============================================================
/** This function initializes Tipograf.
 *  It checks if a config file ~/.tipografrc
 *  exists and reads out the language setting.
 *  Otherwise it assumes that this is the first
 *  start of the application and displays the
 *  initial setup dialog.
 *  @return false on normal operation
 *  @return true on first start
 */
tipoInit * init()
{
  tipoInit * tiInitVals= new tipoInit;            // Structure for return values used in constructor.
  QString sLangFile    = "";                      // Name of the Qt message file.
  QString sCfgFileName = resolveConfigFileName(); // Name of config file (exits on error).
  QStringList slLanguageTxt;                      // List of supported languages.
  QStringList slGUIStyleTxt;                      // List of supported GUI styles.
  QFile fCfg(sCfgFileName);                       // Config file.
  QString sLine     = "";                         // Line read from config file.
  int iEqualPos     = 0;                          // Position of the equal sign within the line.
  int iConvInt      = 0;                          // Converted int.
  bool bConvOk      = false;                      // Holds the return value of conversions.
  QString sParseErr = appTipo->tr("The following errors did occur\n"
				  "while parsing the configuration\n"
				  "file:\n\n");   // Beginning of the parse error message.
  bool bParseErr    = false;                      // Signals a parse error.

  slLanguageTxt += appTipo->tr("English");
  slLanguageTxt += appTipo->tr("Deutsch");

  slGUIStyleTxt = QStyleFactory::keys();          // All styles supported by this version of Qt.

  /*  ========================================
   *  First start init.
   *  ========================================
   */
  if(!fCfg.exists()) {
    // Ask for language setting.
    firststartImpl fsi(slLanguageTxt,
		       slGUIStyleTxt,
		       0, "fsi", true);
    fsi.exec();
    switch(fsi.getLanguage()) {
    case 0:
      // English is the default.
      break;
    case 1:
      sLangFile = "tipograf_de.qm";
#ifdef TRANSDIR
      transTipo->load(sLangFile, TRANSDIR);
#else
#error TRANSDIR not defined. Translator will not work.
#endif
      appTipo->installTranslator(transTipo);
      break;
    default:
      // Error.
      QMessageBox::critical(0,
			    appTipo->tr("Initialization"),
			    appTipo->tr("The first start dialog returned\n"
					"a language value which is not\n"
					"supported. This is a bug in\n"
					"file init.cc, function init()."));
      exit(-1);
      break;
    }
    // Set GUI style.
    QApplication::setStyle(QStyleFactory::create(slGUIStyleTxt[fsi.getGUIStyle()]));

    tiInitVals->bFirstStart       = true;
    tiInitVals->iTipoLanguageInit = fsi.getLanguage();
    tiInitVals->iGUIStyleInit     = fsi.getGUIStyle();
    tiInitVals->sPathA2psInit     = "a2ps";
    tiInitVals->sPathGvInit       = "gv";
    return(tiInitVals);
  } // end of first start.


  /*  ========================================
   *  Normal start init.
   *  ========================================
   */
  // Unable to open file for reading.
  if(!fCfg.open(IO_ReadOnly)) {
    QMessageBox::critical(0,
			  appTipo->tr("Initialization"),
			  appTipo->tr("File: ")
			  + fCfg.name() +
			  appTipo->tr("\nTipograf is unable to open the file\n"
				      "for reading. This means Tipograf cannot\n"
				      "load its settings from that file. The\n"
				      "directory part of the path is supposed to\n"
				      "be your home directory. If this is not the\n"
				      "case you must set the \"HOME\" variable to\n"
				      "a reasonable value in order to run\n"
				      "Tipograf successfully.\n\n"
				      "In a BASH or SH shell you can\n"
				      "accomplish this by the following\n"
				      "command:\n"
				      "export HOME=/path/to/my/directory\n\n"
				      "In a CSH or TCSH shell you can\n"
				      "accomplish this by the following\n"
				      "command:\n"
				      "setenv HOME /path/to/my/directory\n\n"
				      "After setting the \"HOME\" variable\n"
				      "come back and restart Tipograf."));
    exit(-1);
  }
  // File opened successfully.
  QTextStream ts(&fCfg);
  // Parse file.
  bParseErr = false;
  while(!ts.eof()) {
    sLine = ts.readLine();
    if(sLine.contains("=")) {
      iEqualPos = sLine.find("=");

      // Extract TipoLanguage.
      if(sLine.contains("cbTipoLanguage")) {
	sLine = sLine.remove(0, iEqualPos+1);
	iConvInt = sLine.toInt(&bConvOk, 10);
	if(bConvOk == true) {
	  if(iConvInt==0 || iConvInt==1) {
	    if(iConvInt==1) {
	      sLangFile = "tipograf_de.qm";
#ifdef TRANSDIR
	      transTipo->load(sLangFile, TRANSDIR);
#else
	      // This should never happen...
	      transTipo->load(sLangFile, ".");
#endif
	      appTipo->installTranslator(transTipo);
	    }
	  } else {
	    bParseErr = true;
	    sParseErr += appTipo->tr("Tipo language index out of range.\n");
	  }
	} else {
	  bParseErr = true;
	  sParseErr += appTipo->tr("Tipo language index is not a number.\n");
	}
      }

    } // end if contains '='
  } // end while
  
  // Close file.
  fCfg.close();

  // Check for parse error.
  if(bParseErr) {
    QMessageBox::critical(0,
			  appTipo->tr("Initialization"),
			  sParseErr);
    exit(-1);
  }
  
  tiInitVals->bFirstStart       = false;
  tiInitVals->iTipoLanguageInit = 0;
  tiInitVals->iGUIStyleInit     = 0;
  tiInitVals->sPathA2psInit     = "";
  tiInitVals->sPathGvInit       = "";
  return(tiInitVals);
}

// ============================================================
/** Figures out absolute path to config file.
 *  It checks the HOME env variable via getenv.
 *  In case getenv returns an error it kills the
 *  program since it does not make sense to run 
 *  user applications without HOME being set.
 *  @return $HOME/.tipografrc
 */
QString resolveConfigFileName()
{
  QString sConfigFileName = "";
  // Query HOME env variable.
  if(!getenv("HOME")) {
    QMessageBox::critical(0,
			  appTipo->tr("Resolve config file name"),
			  appTipo->tr("The environment variable \"HOME\"\n"
				      "appears not to be set. This means\n"
				      "Tipograf is not able to identify\n"
				      "your home directory in which it must\n"
				      "place its configuration file. You\n"
				      "must set the \"HOME\" variable to a\n"
				      "reasonable value in order to run\n"
				      "Tipograf.\n\n"
				      "In a BASH or SH shell you can\n"
				      "accomplish this by the following\n"
				      "command:\n"
				      "export HOME=/path/to/my/directory\n\n"
				      "In a CSH or TCSH shell you can\n"
				      "accomplish this by the following\n"
				      "command:\n"
				      "setenv HOME /path/to/my/directory\n\n"
				      "After setting the \"HOME\" variable\n"
				      "come back and restart Tipograf."));
    exit(-1);
  }
  QString sHomeVal = getenv("HOME");
  sConfigFileName  = sHomeVal + "/.tipografrc";
  return(sConfigFileName);
}
